/*
 * main.h
 *
 *  Created on: May 15, 2022
 *      Author: bobsc
 */


#ifndef MAIN_H_
#define MAIN_H_


typedef unsigned int  WORD;             //LSH 2-byte Integer 0 - 65,535
typedef unsigned long ULONG;            //LSH 4-byte Interger 0 - 2 billion

//structs

typedef struct {
    unsigned char    hrs;
    unsigned char    mins;
    unsigned char    secs;
} TIME;



// constants

//ASCII CONTROL CHAR
#define     NULL            0x00
#define     SOH             0x01
#define     STX             0x02
#define     ETX             0x03
#define     EOT             0x04
#define     ENQ             0x05
#define     ACK             0x06
#define     CR              0x0D
#define     LF              0x0A
#define     DLE             0x10
#define     DC1             0x11
#define     NAK             0x15
#define     ETB             0x17
#define     ENDSEN          0x3F //"?"
#define     STARTSEN        0X3B  //";"
#define     FIELDSEP        0x3D  //"="
#define     DEL             0x7F

//OTHER CONSTS

#define     KBINMASK        0xF0
#define     CRMASK          0x0F
#define     BF              0x80  //display busy flag
#define     ADMASK          0x0F  //for reading address bits
#define     MASKINT         0x0F  //for converting from ascii to int
#define     txBfr485Len     40    //size of 485 tx buffer
#define     rxBfr485Len     50    //size of 485 rx buffer
#define     DispMax         16    //max size of display buffer

//indexes  location in tx and rx buffers
#define     AdIdx           0x01
#define     CmdIdx          0x03
#define     DvcIdx          0x05
#define     DataIdx          0x07


//#define     MSG0[16]          " ENTER/SWIPE ID "

// flags
//ready flags

#define     KBDATA          0x01
#define     KBREADY         0x02
#define     CRREADY         0X04

//rx buffer flags

#define     ETBrxd          0x01
#define     EOTrxd          0x02

//CPU PORT DEFINES

//PORT 1
#define     POWERIN         0x01    //P1-0
#define     CLS_            0x02
#define     CRTXD           0x04
#define     CRRXD           0x08
#define     CRRTD           0x08
#define     CRCTS           0x10
#define     CRRCL           0x10
#define     TEST7           0x20
#define     DCD             0x40
#define     KBINT           0x80

//port2
#define     C0              0x01  //pulsers
#define     C1              0x02
#define     C2              0x04
#define     C3              0x08
#define     C4              0x10
#define     C5              0x20
#define     TEST6           0x40
#define     TEST5           0x80

//PORT3
#define     D0              0x01  //databus
#define     D1              0x02
#define     D2              0x04
#define     D3              0x08
#define     D4              0x10
#define     D5              0x20
#define     D6              0x40
#define     D7              0x80

//port4
#define     LCDEN_          0x01
#define     RS_             0x02
#define     LCDWrR          0x04
#define     TXD             0x08
#define     RXD             0x10
#define     TXrRX           0x20
#define     PWRFAIL_        0x40
#define     BEEP            0x80

//port5
#define     R0              0x01  //RELAYS
#define     R1              0x02
#define     R2              0x04
#define     RTCCSL          0x08
#define     RTCSDA          0x10
#define     R3              0x20
#define     R4              0x40
#define     R5              0x80

//port6
#define     KBO0            0x01  //KEYBOARD INS/OUTS
#define     KBO1            0x02
#define     KBO2            0x04
#define     KBO3            0x08
#define     KBI0            0x10
#define     KBI1            0x20
#define     KBI2            0x40
#define     KBI3            0x80

//port7
#define     PTXD            0x01  //ptr serial
#define     PRXD            0x02
#define     PTXrRX          0x04
#define     TEST4           0x08
#define     P0              0x10  //pump handles
#define     P1              0x20
#define     P2              0x40
#define     P3              0x80

//port8
#define     P4              0x01
#define     P5              0x02
#define     TEST3           0x04
#define     TEST2           0x08
#define     CRRTS_          0x10
#define     MSDA            0x20
#define     MSCL            0x40
#define     TEST8           0x80

//port9
#define     SPEED0          0x01  //SWITCHES  speed bit0
#define     SPEED1          0x02    //speed bit1
#define     RS485           0x04  //RS485 or RS232 (low)
#define     SWIPE_          0x08  //low for swipe reader, high for RS232

//port J
#define     TDO             0x01  /jtag pins
#define     TDI             0x02
#define     TMS             0x04
#define     TCK             0x08
#define     TEST1           0x10  //test pins (unused)
#define     TEST9           0x20
#define     HFXIN           0x40  //8mhz xytl
#define     HFXOUT          0x80

// key codes defined

#define     KBCLR           0xBE  //pins 7-3 shorted
#define     KB0             0xBD  //pins 7-4
#define     KBENT           0xBB  //pins 7-5
#define     KB1             0x7E  //pins 8-3
#define     KB2             0x7D  //pins 8-4
#define     KB3             0x7B  //pins 8-5
#define     KB4             0xDE  //pins 2-3
#define     KB5             0xDD  //pins 2-4
#define     KB6             0xDB  //pins 2-5
#define     KB7             0xEE  //pins 1-3
#define     KB8             0xED  //pins 1-4
#define     KB9             0xEB  //pins 1-5

//routines

void    main(void);
void    init_micro(void);
void    msTime(void);
void    tenthTime(void);
void    sendStuff(void);
void    ckBusy(void);
void    init_display(void);
void    loadDisp(char bfr[], int posmax);
void    beep(int lenOn, int lenOff, int num);
int     clrBfr (unsigned char buf[], int len);
unsigned char ckStatus(void);
int     BuildStatusResponse(void);
void    sendResponse(void);
int     BuildRSResponse(void);
int     BuildKBDataResponse(void);
int     BuildEEResponse(unsigned char rsp, unsigned char code);
int     BuildCRDataResponse(void);
unsigned short getcrc(unsigned char bfr[], short flen, unsigned short s);


#endif /* MAIN_H_ */
